package	jp.co.sds_corp.fixconnect.transport;

import	java.io.FileInputStream;
import	java.io.IOException;
import	java.security.GeneralSecurityException;
import	java.security.KeyStore;

import	javax.net.ssl.KeyManagerFactory;
import	javax.net.ssl.SSLContext;
import	javax.net.ssl.TrustManagerFactory;

import	jp.co.sds_corp.fixconnect.FIXException;
import	jp.co.sds_corp.fixconnect.message.IMessageReceiver;
import	jp.co.sds_corp.fixconnect.message.MessageReceiveEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public abstract class AbstractTransporter
	implements IMessageReceiver, IStateChangeListener, ITransportEventListener
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	ReceiverSupport recvSupport = new ReceiverSupport();
	TransportSupport transportSupport = new TransportSupport();
	StateSupport stateSupport = new StateSupport();

	public void init( IConfiguration config ) throws Exception {
		socketTimeout = 1000;
		recvBufSize = 10240;
		IConfiguration attr = config.getAttributes();

		host = attr.get( "host" );
		String var;
		var = attr.get( "port" );
		port = Integer.valueOf( var ).intValue();

		var = attr.get( "recvBufSize", null );
		if ( var != null && var.length() > 0 ) {
			recvBufSize = Integer.valueOf( var ).intValue();
		}
		var = attr.get( "socketTimeout", null );
		if ( var != null && var.length() > 0 ) {
			socketTimeout = Integer.valueOf( var ).intValue();
		}

		var = attr.get( "protocol", null );
		if ( var != null && var.length() > 0 ) {
			setProtocol( var );
		}
		var = attr.get( "keyStore", null );
		if ( var != null && var.length() > 0 ) {
			setSsl( true );
			setKeyStore( var );
		}
		var = attr.get( "keyStorePassword", null );
		if ( var != null ) {
			setKeyStorePassword( var );
		}
		var = attr.get( "keyStoreType", null );
		if ( var != null && var.length() > 0 ) {
			setKeyStoreType( var );
		}
		var = attr.get( "keyStoreAlgorithm", null );
		if ( var != null && var.length() > 0 ) {
			setKeyStoreAlgorithm( var );
		}
		var = attr.get( "trustStore", null );
		if ( var != null && var.length() > 0 ) {
			setSsl( true );
			setTrustStore( var );
		}
		var = attr.get( "trustStorePassword", null );
		if ( var != null ) {
			setTrustStorePassword( var );
		}
		var = attr.get( "trustStoreTyp", null );
		if ( var != null && var.length() > 0 ) {
			setTrustStoreType( var );
		}
		var = attr.get( "trustStoreAlgorithm", null );
		if ( var != null && var.length() > 0 ) {
			setTrustStoreAlgorithm( var );
		}
	}

	public void addMessageReceiver( IMessageReceiver l ) {
		recvSupport.addMessageReceiver( l );
	}
	public boolean removeMessageReceiver( IMessageReceiver l ) {
		return recvSupport.removeMessageReceiver( l );
	}
	public void addMessageReceiver( String msgType, IMessageReceiver l ) {
		recvSupport.addMessageReceiver( msgType, l );
	}
	public boolean removeMessageReceiver( String msgType, IMessageReceiver l ) {
		return recvSupport.removeMessageReceiver( msgType, l );
	}
	public void fireMessageReceived( MessageReceiveEvent ev ) {
		recvSupport.fireMessageReceived( ev );
	}
	public void onMessage( MessageReceiveEvent ev ) {
		fireMessageReceived( ev );
	}

	public void addStateChangeListener( IStateChangeListener l ) {
		stateSupport.addStateChangeListener( l );
	}
	public boolean removeStateChangeListener( IStateChangeListener l ) {
		return stateSupport.removeStateChangeListener( l );
	}
	public void fireStateChanged( FIXSession session,
			SessionState oldState, SessionState newState )
	{
		stateSupport.fireStateChanged( session, oldState, newState );
	}
	public void fireStateChanged( StateChangeEvent ev ) {
		stateSupport.fireStateChanged( ev );
	}
	public void stateChanged( StateChangeEvent ev ) {
		fireStateChanged( ev );
	}

	public void addEventListener( ITransportEventListener l ) {
		transportSupport.addEventListener( l );
	}
	public boolean removeEventListener( ITransportEventListener l ) {
		return transportSupport.removeEventListener( l );
	}
	public void fireEventOccurred( TransportEvent ev ) {
		transportSupport.fireEventOccurred( ev );
	}
	public void fireExceptionOccurred( FIXException ex ) {
		transportSupport.fireExceptionOccurred( ex );
	}

	public void onEvent( TransportEvent ev ) {
		fireEventOccurred( ev );
	}
	public void onException( FIXException ex ) {
		fireExceptionOccurred( ex );
	}

	protected SSLContext createSSLContext()
		throws IOException, GeneralSecurityException, Exception
	{
		String msg = "create SSL Context ";
		KeyManagerFactory keyFactory = null;
		if ( this.keyStore != null && this.keyStore.length() > 0 ) {
			String	keyType = getStoreType( keyStoreType );
			KeyStore keyStore = KeyStore.getInstance( keyType );
			char[] keyPass = null;
			if ( keyStorePassword != null && keyStorePassword.length() > 0 ) {
				keyPass = keyStorePassword.toCharArray();
			}
			keyStore.load( new FileInputStream( this.keyStore ), keyPass );

			String keyAlg = getAlgorithm( keyStoreAlgorithm );
			if ( keyAlg == null ) {
				keyAlg = KeyManagerFactory.getDefaultAlgorithm();
			}
			keyFactory = KeyManagerFactory.getInstance( keyAlg );
			keyFactory.init( keyStore, keyPass );

			msg += "KeyStore=" + this.keyStore
					+ "(" + keyType + "," + keyAlg + ") ";
		}
		if ( keyFactory == null ) {
			throw( new Exception( "can't create KeyManagerFactory" ));
		}

		TrustManagerFactory trustFactory = null;
		if ( this.trustStore != null && this.trustStore.length() > 0 ) {
			String	trustType = getStoreType( trustStoreType );
			KeyStore trustStore = KeyStore.getInstance( trustType );
			char[] trustPass = null;
			if ( trustStorePassword != null && trustStorePassword.length() > 0 ) {
				trustPass = trustStorePassword.toCharArray();
			}
			trustStore.load( new FileInputStream( this.trustStore ), trustPass );

			String trustAlg = getAlgorithm( trustStoreAlgorithm );
			if ( trustAlg == null ) {
				trustAlg = TrustManagerFactory.getDefaultAlgorithm();
			}
			trustFactory = TrustManagerFactory.getInstance( trustAlg );
			trustFactory.init( trustStore );
			msg += "TrustStore=" + this.trustStore
					+ "(" + trustType + "," + trustAlg + ") ";
		}
		if ( trustFactory == null ) {
			throw( new Exception( "can't create TrustManagerFactory" ));
		}

		String protocol = this.protocol;
		if ( protocol == null || protocol.length() == 0 ) {
			protocol = "SSL";
		}
		msg += "protocol=" + protocol;
		logger.debug( this, msg );

		SSLContext context = SSLContext.getInstance( protocol );
		context.init( keyFactory.getKeyManagers(),
						trustFactory.getTrustManagers(), null );
		return context;
	}

	private String getStoreType( String var ) {
		String ret;
		if ( var != null && var.length() > 0 ) {
			ret = var;
		} else {
			ret = "jks";
		}
		return ret;
	}

	private String getAlgorithm( String var ) {
		String ret = null;
		if ( var != null && var.length() > 0 ) {
			ret = var;
		}
		return ret;
	}

	String	host;	// ホスト名
	int	port;	// ポート番号
	int	recvBufSize;	// 受信バッファサイズ
	int	socketTimeout;	// Socketタイムアウト
	boolean	ssl;	// SSLフラグ
	String	protocol;	// プロトコル
	String	keyStore;	// キーストア
	String	keyStorePassword;	// キーストアパスワード
	String	keyStoreType;	// キーストアタイプ
	String	keyStoreAlgorithm;	// キーストアのアルゴリズム
	String	trustStore;	// トラストストア
	String	trustStorePassword;	// トラストストアパスワード
	String	trustStoreType;	// トラストストアタイプ
	String	trustStoreAlgorithm;	// トラストストアのアルゴリズム

	/**
	* ホスト名を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var;
	}
	/**
	* ホスト名を取得します。
	* @return	ホスト名
	*/
	public String getHost() {
		return host;
	}

	/**
	* ポート番号を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var;
	}
	/**
	* ポート番号を取得します。
	* @return	ポート番号
	*/
	public int getPort() {
		return port;
	}

	/**
	* 受信バッファサイズを設定します。
	* @param	var		設定するデータ
	*/
	public void setRecvBufSize( int var ) {
		this.recvBufSize = var;
	}
	/**
	* 受信バッファサイズを取得します。
	* @return	受信バッファサイズ
	*/
	public int getRecvBufSize() {
		return recvBufSize;
	}

	/**
	* Socketタイムアウトを設定します。
	* @param	var		設定するデータ
	*/
	public void setSocketTimeout( int var ) {
		this.socketTimeout = var;
	}
	/**
	* Socketタイムアウトを取得します。
	* @return	Socketタイムアウト
	*/
	public int getSocketTimeout() {
		return socketTimeout;
	}

	/**
	* SSLフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setSsl( boolean var ) {
		this.ssl = var;
	}
	/**
	* SSLフラグを取得します。
	* @return	SSLフラグ
	*/
	public boolean isSsl() {
		return ssl;
	}

	/**
	* プロトコルを設定します。
	* @param	var		設定するデータ
	*/
	public void setProtocol( String var ) {
		this.protocol = var;
	}
	/**
	* プロトコルを取得します。
	* @return	プロトコル
	*/
	public String getProtocol() {
		return protocol;
	}

	/**
	* キーストアを設定します。
	* @param	var		設定するデータ
	*/
	public void setKeyStore( String var ) {
		this.keyStore = var;
	}
	/**
	* キーストアを取得します。
	* @return	キーストア
	*/
	public String getKeyStore() {
		return keyStore;
	}

	/**
	* キーストアパスワードを設定します。
	* @param	var		設定するデータ
	*/
	public void setKeyStorePassword( String var ) {
		this.keyStorePassword = var;
	}
	/**
	* キーストアパスワードを取得します。
	* @return	キーストアパスワード
	*/
	public String getKeyStorePassword() {
		return keyStorePassword;
	}

	/**
	* キーストアタイプを設定します。
	* @param	var		設定するデータ
	*/
	public void setKeyStoreType( String var ) {
		this.keyStoreType = var;
	}
	/**
	* キーストアタイプを取得します。
	* @return	キーストアタイプ
	*/
	public String getKeyStoreType() {
		return keyStoreType;
	}

	/**
	* キーストアのアルゴリズムを設定します。
	* @param	var		設定するデータ
	*/
	public void setKeyStoreAlgorithm( String var ) {
		this.keyStoreAlgorithm = var;
	}
	/**
	* キーストアのアルゴリズムを取得します。
	* @return	キーストアのアルゴリズム
	*/
	public String getKeyStoreAlgorithm() {
		return keyStoreAlgorithm;
	}

	/**
	* トラストストアを設定します。
	* @param	var		設定するデータ
	*/
	public void setTrustStore( String var ) {
		this.trustStore = var;
	}
	/**
	* トラストストアを取得します。
	* @return	トラストストア
	*/
	public String getTrustStore() {
		return trustStore;
	}

	/**
	* トラストストアパスワードを設定します。
	* @param	var		設定するデータ
	*/
	public void setTrustStorePassword( String var ) {
		this.trustStorePassword = var;
	}
	/**
	* トラストストアパスワードを取得します。
	* @return	トラストストアパスワード
	*/
	public String getTrustStorePassword() {
		return trustStorePassword;
	}

	/**
	* トラストストアタイプを設定します。
	* @param	var		設定するデータ
	*/
	public void setTrustStoreType( String var ) {
		this.trustStoreType = var;
	}
	/**
	* トラストストアタイプを取得します。
	* @return	トラストストアタイプ
	*/
	public String getTrustStoreType() {
		return trustStoreType;
	}

	/**
	* トラストストアのアルゴリズムを設定します。
	* @param	var		設定するデータ
	*/
	public void setTrustStoreAlgorithm( String var ) {
		this.trustStoreAlgorithm = var;
	}
	/**
	* トラストストアのアルゴリズムを取得します。
	* @return	トラストストアのアルゴリズム
	*/
	public String getTrustStoreAlgorithm() {
		return trustStoreAlgorithm;
	}

}
